@@ -530,15 +530,14 @@ from django.utils.safestring import mark_safe
 class DataTransform(forms.ModelForm):
     id = forms.CharField(required=False,
                          widget=forms.HiddenInput(attrs={'class': 'form-control'}))
-    accessory = forms.ModelChoiceField(required=True,queryset=Accessory.objects.all(),
-                                widget=forms.HiddenInput(attrs={'class': 'form-control'}))
+    accessory = forms.ModelChoiceField(required=True, queryset=Accessory.objects.all(),
+                                       widget=forms.HiddenInput(attrs={'class': 'form-control'}))
 
-    parent = forms.ModelChoiceField(required=True,queryset=DataTransformer.objects.all(),
-                             widget=forms.HiddenInput(attrs={'class': 'form-control'}))
+    parent = forms.ModelChoiceField(required=True, queryset=DataTransformer.objects.all(),
+                                    widget=forms.HiddenInput(attrs={'class': 'form-control'}))
 
     type = forms.ModelChoiceField(required=True, queryset=DataTransformerTypes.objects.all())
 
-
     def __init__(self, *args, **kwargs):
         print('Init Data Transform')
         super(DataTransform, self).__init__(*args, **kwargs)
@@ -546,7 +545,7 @@ class DataTransform(forms.ModelForm):
         #     DataTransformerTypes.objects.all().values_list('type', 'type'))
         # self.fields['type'] = ChoiceField(choices=tuple(types))
         try:
-            if hasattr(self.instance, 'get_children') and self.instance.get_children().count() > 0:
+            if hasattr(self.instance, 'get_children') and self.instance.get_children().count() >= 0:
                 print('Adding to child form node: ' + str(self.instance.get_children().count() + 1))
 
                 dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer,
@@ -557,8 +556,9 @@ class DataTransform(forms.ModelForm):
                     queryset=self.instance.get_children(),
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
-                    initial=[{'parent': self.instance, 'accessory':self.instance.accessory, 'id':str(uuid.uuid4())}]
+                    initial=[{'parent': self.instance, 'accessory': self.instance.accessory, 'id': str(uuid.uuid4())}]
                 )
+
             else:
                 dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer,
                                                                      fk_name='parent', form=DataTransform,
@@ -572,19 +572,25 @@ class DataTransform(forms.ModelForm):
             self.nested_constants_form = DataConstantsTransformerInlineFormset(
                 instance=self.instance,
                 data=self.data if self.is_bound else None,
-                files=self.files if self.is_bound else None
+                files=self.files if self.is_bound else None,
+                initial=[{'data_transformer': self.instance}]
+
             )
 
             self.nested_hardware_outputs_form = HardwareTransformerInlineFormset(
                 instance=self.instance,
                 data=self.data if self.is_bound else None,
-                files=self.files if self.is_bound else None
+                files=self.files if self.is_bound else None,
+                initial=[{'data_transformer': self.instance}]
+
             )
 
             self.nested_hardware_stats_form = HardwareStatsTransformerInlineFormset(
                 instance=self.instance,
                 data=self.data if self.is_bound else None,
-                files=self.files if self.is_bound else None
+                files=self.files if self.is_bound else None,
+                initial=[{'data_transformer': self.instance}]
+
             )
         except Exception:
             print('excpt')
@@ -613,15 +619,21 @@ class DataTransform(forms.ModelForm):
     def save(self, commit=True):
         print('node')
         print('save: ' + str(self.instance.id))
-        super(DataTransform, self).save(commit)
-        if hasattr(self, 'nested_constants_form') and self.nested_constants_form is not None:
+        root_node = super(DataTransform, self).save(commit)
+        if hasattr(self,
+                   'nested_constants_form') and self.nested_constants_form is not None and self.nested_constants_form.is_valid():
             self.nested_constants_form.save(commit)
-        if hasattr(self, 'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:
+        if hasattr(self,
+                   'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None and self.nested_hardware_outputs_form.is_valid():
             self.nested_hardware_outputs_form.save(commit)
-        if hasattr(self, 'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None:
+        if hasattr(self,
+                   'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None and self.nested_hardware_stats_form.is_valid():
             self.nested_hardware_stats_form.save(commit)
-        if hasattr(self, 'nested_data_transformer_form') and self.nested_data_transformer_form is not None:
+        if hasattr(self,
+                   'nested_data_transformer_form') and self.nested_data_transformer_form is not None and self.nested_data_transformer_form.is_valid():
             self.nested_data_transformer_form.save(commit)
+        root_node.refresh_from_db()
+        return root_node
 
     class Meta:
         model = DataTransformer
@@ -657,7 +669,7 @@ class DataTransformRoot(forms.ModelForm):
                     queryset=self.instance.get_children(),
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
-                    initial=[{'parent': self.instance, 'accessory':self.instance.accessory, 'id':str(uuid.uuid4())}]
+                    initial=[{'parent': self.instance, 'accessory': self.instance.accessory, 'id': str(uuid.uuid4())}]
                 )
             else:
                 dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer,
@@ -675,19 +687,25 @@ class DataTransformRoot(forms.ModelForm):
         self.nested_constants_form = DataConstantsTransformerInlineFormset(
             instance=self.instance,
             data=self.data if self.is_bound else None,
-            files=self.files if self.is_bound else None
+            files=self.files if self.is_bound else None,
+            initial=[{'data_transformer': self.instance}]
+
         )
 
         self.nested_hardware_outputs_form = HardwareTransformerInlineFormset(
             instance=self.instance,
             data=self.data if self.is_bound else None,
-            files=self.files if self.is_bound else None
+            files=self.files if self.is_bound else None,
+            initial=[{'data_transformer': self.instance}]
+
         )
 
         self.nested_hardware_stats_form = HardwareStatsTransformerInlineFormset(
             instance=self.instance,
             data=self.data if self.is_bound else None,
-            files=self.files if self.is_bound else None
+            files=self.files if self.is_bound else None,
+            initial=[{'data_transformer': self.instance}]
+
         )
 
     def as_p(self):
@@ -709,20 +727,24 @@ class DataTransformRoot(forms.ModelForm):
         forms = forms + '</div></div>'
         return mark_safe(forms)
 
-
-
     def save(self, commit=True):
         print('root')
         print('save: ' + str(self.instance.id))
-        super(DataTransformRoot, self).save(commit)
-        if hasattr(self, 'nested_constants_form') and self.nested_constants_form is not None:
+        root_node = super(DataTransformRoot, self).save(commit)
+        if hasattr(self,
+                   'nested_constants_form') and self.nested_constants_form is not None and self.nested_constants_form.is_valid():
             self.nested_constants_form.save(commit)
-        if hasattr(self, 'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:
+        if hasattr(self,
+                   'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None and self.nested_hardware_outputs_form.is_valid():
             self.nested_hardware_outputs_form.save(commit)
-        if hasattr(self, 'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None:
+        if hasattr(self,
+                   'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None and self.nested_hardware_stats_form.is_valid():
             self.nested_hardware_stats_form.save(commit)
-        if hasattr(self, 'nested_data_transformer_form') and self.nested_data_transformer_form is not None:
+        if hasattr(self,
+                   'nested_data_transformer_form') and self.nested_data_transformer_form is not None and self.nested_data_transformer_form.is_valid():
             self.nested_data_transformer_form.save(commit)
+        root_node.refresh_from_db()
+        return root_node.get_root()
 
     class Meta:
         model = DataTransformer
